import java.awt.Color;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * 
 * A class that represent multiple views of the simulation. 
 * This class can forward calls to other views.
 * 
 * @author Poul Henriksen
 * @version 2005.03.15
 */
public class MultiView  implements SimulatorView
{
    Set views;
    
    public MultiView()
    {
        views = new HashSet();
    }
   
    /**
     * Define a color to be used for a given class of animal.
     */
    public void setColor(Class animalClass, Color color)
    {
        Iterator iter = views.iterator();
        while(iter.hasNext()) {
            ((SimulatorView) iter.next()).setColor(animalClass, color);
        }
    } 

    /**
     * Show the current status of the field.
     * @param step Which iteration step it is.
     * @param field The field whose status is to be displayed.
     */
    public void showStatus(int step, Field field)
    {
        Iterator iter = views.iterator();
        while(iter.hasNext()) {
            ((SimulatorView) iter.next()).showStatus(step, field);
        }
    }

    /**
     * Determine whether the simulation should continue to run.
     * 
     * This method only forward to the first view
     * 
     * @return true If there is more than one species alive.
     */
    public boolean isViable(Field field)
    {
        Iterator iter = views.iterator();
        if(iter.hasNext()) {
            return ((SimulatorView) iter.next()).isViable(field);
        } else {
            return false;
        }
    }
    
    public void addView(SimulatorView view) 
    {
        views.add(view);
    }
}
